''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'                                                                            '
'    SIMULATION OF THE ACTION POTENTIAL OF THE SQUID GIANT AXON              '
'         AFTER HODGKIN & HUXLEY, J. PHYSIOL. 117:500 (1952).                '
'                                                                            '
'    Computer program based on Randall JE, Microcomputers                    '
'         and Physiological Simulation, Addison-Wesley,                      '
'         Reading, MA, pp. 157-198 (1980).                                   '
'                                                                            '
'    Written by Stephen W. Jones, 2/16/87; revised 2/25/89.                  '
'    Written for IBM PC-compatibles, using QuickBASIC 4.5                    '
'    Requires EGA, CGA, Hercules, or AT&T graphics adapter.                  '
'                                                                            '
'    CHANNEL.BAS                                                             '
'                                                                            '
'    This version simulates the behavior of single Na and K                  '
'         channels, using the rate constants from the                        '
'         Hodgkin-Huxley model and random numbers.                           '
'         At the start of each record, the activation                        '
'         gates are assumed to be closed, and the Na                         '
'         channel inactivation gate is open, as though                       '
'         the holding potential had been very negative.                      '
'    The behavior of the individual gates and of the channel                 '
'         are shown for each sweep, and a "pseudomacroscopic"                '
'         conductance is constructed by adding channel openings              '
'         from different sweeps.                                             '
'                                                                            '
'    This program will return control to a menu program that calls           '
'         it via CHAIN.                                                      '
'                                                                            '
'    Other modules needed:  EDITVAC, PARAMC                                  '
'                                                                            '
'    Include files:                                                          '
'         DECLARES.INC (subprogram declarations and common blocks)           '
'         GRAPHICS.INC (subroutine Graphics and error traps)                 '
'                                                                            '
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

'    $INCLUDE: 'declares.inc'
    
	DECLARE SUB ClearTop ()       'Subprograms for CHANNEL only
	DECLARE SUB ClearMiddle ()
	DECLARE SUB PrintOptions ()
	DECLARE SUB RightLabels ()
	DECLARE SUB AskPause ()
    
	COMMON /ChannelOnly/ Pause

		True = -1
		False = NOT (True)  'Allow logical if.
    
	CALL PeekInfo            'get simulation type (not really used here)
	GOSUB Graphics           'find monitor/adapter type, initialize graphics
	GOSUB Dimensions         'dimension variables
	CALL PrintIntroText      'introductory text

INITIAL:
	GOSUB Initialize         'default parameters, etc.
	CALL Parameters          'show default parameters

NEXTRUN:
	CALL NewParameters       'edit parameters; allow exit from program
    
		IF ReInit THEN GOTO INITIAL        'Reset parameters to defaults
		IF Pause THEN
			Pause = False
			CALL RightLabels              'Continue interrupted run
			GOTO RESUMERUN
		END IF

	GOSUB NewVariables       'get parameters from arrays

		SweepNum = 1             'first sweep of run
		ON ERROR GOTO TRAP       'abort run if error

	GOSUB Rates              'get rate constants
	GOSUB InitializeRun      'initialize variables
	GOSUB Labels             'draw and label axes

NEWSWEEP:
	CALL AskPause            'allow pause after each sweep
	GOSUB Gating             'calculate & plot gate openings/closings
	GOSUB PlotChannel        'calculate & plot channel openings/closings,
						'calculate & plot macroscopic conductance

		Pause$ = INKEY$                         'Check this each sweep:
		IF LEN(Pause$) <> 0 THEN                     'if key hit to pause,
			Pause = True
			DO
			LOOP WHILE INKEY$ <> ""                 'flush keyboard buffer,
			GOTO NEXTRUN                            'get next request.
		END IF
	   
RESUMERUN:
		SweepNum = SweepNum + 1                      'increment sweep counter
		IF SweepNum <= Sweeps THEN
			GOTO NEWSWEEP                           'another sweep
		ELSE GOTO NEXTRUN                            'or another run
		END IF

'  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '

TRAP:
	ErrorType = ERR                         'Main error trap
	IF ErrorType = 6 THEN                        'Error 6 = overflow
		SOUND 500, 1                            'Sound a warning
		RESUME NEXTRUN
	END IF
	ON ERROR GOTO 0

'    $INCLUDE: 'graphics.inc'

Dimensions:
	Points = 500 / XScale                   '500 calculation intervals
    
	DIM gK(Points)
	DIM gNa(Points)
	DIM channel(Points)
	DIM Current(Points)
	DIM Par(10)

	RETURN

Initialize:
	Sweeps = 20             'defaults:      20 sweeps
	Type$ = "K"                             'default = K channel
	PrintType = 1                                '1 = K, 2 = Na channel
	mV = 0                                  'default command potential
    
	IF Ega AND Mono THEN               'switch "colors" on monochrome
		Green = 3                          '3 = bright white
		Red = 1                            '1 = low intensity white
	END IF
									
	Opened = 1                         'open channel = 1, closed = 0
	Closed = 0
    
	Pause = False                      'true when a run is interrupted
	ReInit = False                     'true if parameters to be initialized

	PrintDefaultAt = 14                'used for parameter display
	SidebarCol = 69
	Blanks$ = "            "
    
	Parameter(2, 1) = Sweeps           'Read parameters into array
	Parameter(2, 2) = mV
	Parameter(2, 3) = PrintType

	RETURN

NewVariables:
	Sweeps = Parameter(2, 1)           'Read parameters from array
	mV = Parameter(2, 2)
	ChannelType = Parameter(2, 3)
	SELECT CASE ChannelType
		CASE 1
			Type$ = "K"
		CASE ELSE
			Type$ = "Na"
	END SELECT

	RETURN

Rates:
	V = -mV - 60                       'set rp = -60 mV.  V is in H-H units
	IF V + 25 = 0 THEN V = -25.01           'avoid division by zero
	IF V + 10 = 0 THEN V = -10.01

	an = .01 * (V + 10) / (EXP((V + 10) / 10) - 1)    'a for opening rate
	am = .1 * (V + 25) / (EXP((V + 25) / 10) - 1)
	ah = .07 * EXP(V / 20)
	bn = .125 * EXP(V / 80)                           'b for closing rate
	bm = 4 * EXP(V / 18)
	bh = 1 / (EXP((V + 30) / 10) + 1)

	RETURN

InitializeRun:
	VIEW                                    'remove any old viewports
	WINDOW

	IF Type$ = "K" THEN
		TTotal = 20                        'note longer record for K
		Gate$ = "n"
		Gate4$ = "n"
	ELSE                                    'change values if Na channel
		TTotal = 5
		Gate$ = "m"
		Gate4$ = "h"
	END IF

	TPoint = Points / TTotal                '# points per msec
	dt = 1 / TPoint                         'calculation interval

	FOR i = 1 TO Points - 1
		Current(i) = Closed                'initialize current to 0
	NEXT i

	RANDOMIZE TIMER                         'reseed random number generator

	RETURN

Labels:
	CLS
	CALL Parameters                         'show new parameter values

	LOCATE 23, 5                            'label time axis
	PRINT "0"
	LOCATE 23, 35
	PRINT TTotal / 2
	LOCATE 23, 67
	PRINT TTotal; "msec  "
	
	XTick = 28 / XScale
	XLeft = 34 / XScale
	XRight = 534 / XScale
    
	Y1 = 160 / YScale
	Y2 = 181 / YScale
	Y3 = 201 / YScale
	Y4 = 231 / YScale
	Y5 = 266 / YScale
	Y6 = 301 / YScale

	LINE (XTick, Y6)-(XRight, Y6)           'X axis for probabilities
	LINE (XLeft, Y4)-(XLeft, Y6)            'Y axis
	LINE (XTick, Y4)-(XLeft, Y4)            'tick mark at p = 0
	LINE (XTick, Y5)-(XLeft, Y5)            'tick mark at p = .5
								    
	LINE (XTick, Y3)-(XRight, Y3)           'X axis for gates
	LINE (XLeft, Y1)-(XLeft, Y3)            'Y axis
	LINE (XTick, Y1)-(XLeft, Y1)            'tick mark at p = 0
	LINE (XTick, Y2)-(XLeft, Y2)            'tick mark at p = .5
	FOR i = 0 TO 10
		XTime = XLeft + i * 50 / XScale
		LINE (XTime, Y6)-(XTime, Y6 + 5 / YScale)    'time marks
	NEXT i

	LOCATE 1, 1
	PRINT "GATES:"
	IF Ega THEN COLOR Blue                  'label gates, channel
	PRINT " "; Gate$                             'activation gates in blue
	PRINT " "; Gate$
	PRINT " "; Gate$
	IF Type$ <> "K" AND Ega THEN COLOR Red
	PRINT " "; Gate4$                            'inactivation gate in red
	PRINT
    
	IF Ega THEN COLOR White
	PRINT "CHANNEL:"
	IF Ega THEN COLOR Yellow                     'channel in yellow
	IF Type$ = "K" THEN
		PRINT "  4"                             'm cubed h, or n to 4th
	ELSE
		PRINT "  3"
	END IF
	IF Type$ = "K" THEN
		PRINT " n  "
	ELSE
		PRINT " m h"
	END IF

	IF Ega THEN COLOR White
	LOCATE 11, 1
	PRINT "PROBABILITY THAT GATE IS OPEN:"       'labels for gates
	PRINT
	IF Type$ = "K" THEN
		IF Ega THEN COLOR Blue
		PRINT " n"
		PRINT "  "
	ELSE
		IF Ega THEN COLOR Red
		PRINT " h"
		IF Ega THEN COLOR Blue
		PRINT " m"
	END IF
	IF Ega THEN COLOR White
	LOCATE 12, 3
	PRINT "1"
	LOCATE 15, 3
	PRINT "0"

	LOCATE 16, 1
	PRINT "PROBABILITY THAT CHANNEL IS OPEN:"
	CALL RightLabels
    
	LOCATE 17, 3
	PRINT "1"
	LOCATE 22, 3
	PRINT "0"

	RETURN
   
Gating:
	INITIAL = Closed                        'gates initially closed (except h)
	FOR j = 1 TO Points - 1
		channel(j) = Opened                'initialize channel to open!
	NEXT j

	IF Ega THEN COLOR White                 'display sweep number
	LOCATE 21, 69
	PRINT "            ";
	LOCATE , 69
	PRINT "Sweep #";
	CALL PrintNum(SweepNum)

	Upper = 15                              'upper plotting limit for 1st gate
	FOR Gate = 1 TO 4                                 'calculate 4 gates
		IF Ega THEN COLOR Blue                       'activation gates blue
		TSum = 0
		WINDOW (0, Closed)-(Points - 1, Opened)      'closed = 0, open = 1
		VIEW (XLeft + 1, Upper / YScale)-(XRight, (Upper + 10) / YScale)
		CLS 1                                        'clear viewport

		IF Type$ = "K" THEN                          'set appropriate rates
			TauClosed = 1 / an
			TauOpen = 1 / bn
		ELSE
			TauClosed = 1 / am
			TauOpen = 1 / bm
			IF Gate = 4 THEN
				TauClosed = 1 / ah
				TauOpen = 1 / bh
				IF Ega THEN COLOR Red              'plot h gate in red
				INITIAL = Opened                   'h gate initially open
			END IF
		END IF

		GOSUB OpenClosed                             'calc & plot gates
		Upper = Upper + 14                           'reset plotting limit
	NEXT Gate

	RETURN

OpenClosed:              'Routine to calculate & plot open and closed gates
	IF INITIAL = Opened GOTO openGate                 'h gate initially open

closedGate:
	Prob = RND                                        'rnd = random, 0 to 1
	TClosed = -1 * TauClosed * LOG(Prob) * TPoint     'calc closed time
	TSumNew = TSum + TClosed                          'time so far in sweep
	IF TSumNew > Points THEN TSumNew = Points         'avoid overflow
	LINE (TSum, Closed)-(TSumNew, Closed)             'draw closed time
	FOR j = TSum TO TSumNew
		IF j < Points THEN channel(j) = Closed       'channel now closed
	NEXT j
	TSum = TSumNew
	LINE (TSum, Closed)-(TSum, Opened)

openGate:
	Prob = RND                                        'as for closed times
	topen = -1 * TauOpen * LOG(Prob) * TPoint
	TSumNew = TSum + topen
	IF TSumNew > Points THEN TSumNew = Points
	LINE (TSum, Opened)-(TSumNew, Opened)
	TSum = TSumNew
	LINE (TSum, Opened)-(TSum, Closed)

	IF TSum < Points THEN GOTO closedGate             'get next closed time

	RETURN

PlotChannel:                            'now plot channel, sum, theoretical g
	VIEW (XLeft + 1, 99 / YScale)-(XRight, 127 / YScale) 'clear viewports
	CLS 1
	VIEW (XLeft + 1, 231 / YScale)-(XRight, 300 / YScale)
	CLS 1
	IF SweepNum = 1 THEN
		VIEW (XLeft + 1, 160 / YScale)-(XRight, 200 / YScale)
		n = Closed                         'The n and m gates are closed
		m = Closed                              'at the start of the sweep,
		h = Opened                              'and the h gate is open.
		CLS 1
	END IF
	VIEW
	WINDOW                                  'Abandon elegance for speed

	FOR K = 1 TO Points - 1
		KPlot = XLeft + 1 + K
		K% = K / 2
		K# = K / 2
		IF SweepNum = 1 THEN GOSUB Gates             'theoretical gates

		IF Ega THEN COLOR Yellow
		IF K% = K# OR HiRes = 9 THEN    'plot theoretical g
			IF Type$ = "K" THEN
				PSET (KPlot, (300 - 69 * gK(K)) / YScale)
			ELSE
				PSET (KPlot, (300 - 69 * gNa(K)) / YScale)
			END IF
		END IF

		PSET (KPlot, (127 - 28 * channel(K)) / YScale)    'plot this channel
		IF channel(K) <> channel(K - 1) THEN
			LINE (KPlot, 99 / YScale)-(KPlot, 127 / YScale)
		END IF

		Current(K) = Current(K) + channel(K)              'plot current
		IF Ega THEN COLOR Green
		LastY = YPlot
		YPlot = (300 - 69 * Current(K) / SweepNum) / YScale
		IF Current(K) <> Current(K - 1) AND K > 1 THEN
			LINE (KPlot, YPlot)-(KPlot, LastY)
		END IF
		PSET (KPlot, (300 - 69 * Current(K) / SweepNum) / YScale)

		IF Ega THEN COLOR White
		IF K < 51 THEN                               'dotted line at "open"
			XSpot = XLeft + K * 10 / XScale
			PSET (XSpot, 231 / YScale)              'for channel prob.
			PSET (XSpot, 160 / YScale)              'for gate prob.
		END IF

	NEXT K

	RETURN

Gates:
	dn = an * (1 - n) - bn * n                   'increments
	dm = am * (1 - m) - bm * m
	dh = ah * (1 - h) - bh * h

	n = n + dn * dt                              'gate p(open)
	m = m + dm * dt
	h = h + dh * dt

	gK(K) = n ^ 4                                'n to 4th.  All open = 1.
	gNa(K) = m ^ 3 * h                           'm cubed h

	IF Ega THEN COLOR Blue
	IF Type$ = "K" THEN                               'plot theoretical gates
		PSET (KPlot, (200 - 40 * n) / YScale)
	ELSE
		PSET (KPlot, (200 - 40 * m) / YScale)
		IF Ega THEN COLOR Red
		IF NOT Mono OR K% = K# THEN                       'If mono, plot
			PSET (KPlot, (200 - 40 * h) / YScale)             'alt. points
		END IF                                                 'for "h".
	END IF

	RETURN

SUB AskPause STATIC
	SHARED Ega, Red, White, SidebarCol, Blanks$

	IF Ega THEN COLOR Red                   'allow pause between sweeps
	LOCATE 11, SidebarCol
	PRINT "HIT ANY KEY "
	LOCATE , SidebarCol
	PRINT " TO PAUSE   "                         'text in red
	LOCATE , SidebarCol
	PRINT " AFTER THIS "
	LOCATE , SidebarCol
	PRINT " SWEEP      "
	LOCATE , SidebarCol
	PRINT Blanks$
	IF Ega THEN COLOR White

END SUB

SUB ClearBottom STATIC                  'Erase bottom right part of screen
	SHARED SidebarCol, Blanks$
	FOR i = 17 TO 22
		LOCATE i, SidebarCol
		PRINT Blanks$
	NEXT i
END SUB

SUB ClearMiddle STATIC                  'Erase middle right part of screen
	SHARED SidebarCol, Blanks$
	FOR i = 10 TO 16
		LOCATE i, SidebarCol
		PRINT Blanks$
	NEXT i
END SUB

SUB ClearTop STATIC                          'Erase top right part of screen
	SHARED SidebarCol, Blanks$
	FOR i = 1 TO 9
		LOCATE i, SidebarCol
		PRINT Blanks$
	NEXT i
END SUB

SUB Parameters
	SHARED Ega, White, Blanks$, Type$, Sweeps, mV, SidebarCol
    
	CALL ClearTop
	IF Ega THEN COLOR White
	LOCATE 1, SidebarCol
	PRINT "Channel type"
	LOCATE , SidebarCol
	PRINT "(Na or K)"
	LOCATE , SidebarCol
	PRINT Blanks$;
	LOCATE , SidebarCol + 2
	PRINT Type$
	LOCATE 4, SidebarCol
	PRINT "Number of"
	LOCATE , SidebarCol
	PRINT "sweeps"
	LOCATE , SidebarCol
	PRINT Blanks$;
	LOCATE , SidebarCol + 2
	CALL PrintNum(Sweeps)
	LOCATE 7, SidebarCol
	PRINT "Voltage (mV)"
	LOCATE , SidebarCol
	PRINT Blanks$;
	LOCATE , SidebarCol + 2
	CALL PrintNum(mV)

END SUB

SUB PrintEditKeys
	SHARED SidebarCol
	SHARED DownArrow$, UpArrow$
	SHARED Simulation%
	SHARED MenuN, OutputMenuN
   
	CALL ClearRight
  
	LOCATE 2, SidebarCol
	PRINT "<" + DownArrow$ + ">  Down"
	LOCATE , SidebarCol
	PRINT "<" + UpArrow$ + ">  Up"
  
	PRINT
	LOCATE , SidebarCol
	PRINT "<Home> for"
	LOCATE , SidebarCol
	PRINT " 1st option"
	LOCATE , SidebarCol
	PRINT "<End> for"
	LOCATE , SidebarCol
	PRINT " last option"
	PRINT
	LOCATE , SidebarCol
	PRINT "<e> to exit "
	LOCATE , SidebarCol
	PRINT " this menu  "
	PRINT
	LOCATE , SidebarCol
	PRINT "<y> or <n>  "
	LOCATE , SidebarCol
	PRINT " to change  "
	LOCATE , SidebarCol
	PRINT " an option  "
	PRINT
	LOCATE , SidebarCol
	PRINT "<Esc> to    "
	LOCATE , SidebarCol
	PRINT " delete line"
	PRINT
	LOCATE , SidebarCol
	PRINT "<r> to run  "

END SUB

SUB PrintIntroText
	SHARED Ega, Blue, Red, White, Yellow, Called$

	LOCATE 1, 8
	IF Ega THEN COLOR Blue
	PRINT "AXOVACS Electrophysiology Computer Lab - Version 2.0"
	LOCATE , 18
	PRINT "(C) 1989 Axon Instruments, Inc."
	IF Ega THEN COLOR Red
	PRINT
	LOCATE , 22
	PRINT "PATCH CLAMP SIMULATION"
	IF Ega THEN COLOR White
	PRINT
	PRINT "YOU can choose to simulate a Na or K channel,"
	PRINT "     enter the number of trials,"
	PRINT "     and choose the command voltage."
	PRINT ""
	PRINT "THE COMPUTER will calculate and display"
	PRINT "     simulated opening and closing of gates,"
	PRINT "     the resulting opening/closing of channels,"
	PRINT "     the behavior of the channel, averaged over"
	PRINT "          several sweeps, and"
	PRINT "     the theoretical probability that the channel"
	PRINT "          is open at a given time, from the"
	PRINT "          Hodgkin-Huxley equations."
	PRINT
	IF Ega THEN COLOR Yellow
	IF Called$ <> "" THEN
		PRINT "<r> to run, <e> to edit, <Esc> to go back to the main menu"
	ELSE PRINT "          <r> to run, <e> to edit, <q> to quit"
	END IF
	IF Ega THEN COLOR White

END SUB

SUB PrintOptions STATIC                 'Options when run is over
	SHARED Ega, Green, White, SidebarCol, Called$
    
	CALL ClearMiddle
	CALL ClearBottom
	IF Ega THEN COLOR Green
	LOCATE 10, SidebarCol
	PRINT " OPTIONS:   "
	IF Ega THEN COLOR White
	LOCATE , SidebarCol
	PRINT "<r>un       "               'start a new run, same parameters
	LOCATE , SidebarCol
	PRINT "<e>dit      "               'edit parameters
	LOCATE , SidebarCol
	PRINT "<i>nitialize"               'initialize
	LOCATE , SidebarCol
	PRINT " all values "
	LOCATE , SidebarCol
	PRINT " to defaults"
	LOCATE , SidebarCol
	IF Called$ = "" THEN               'quit
		PRINT "<q>uit      "
	ELSE PRINT "<Esc> for   "
		LOCATE , SidebarCol
		PRINT " main menu  "
	END IF

END SUB

SUB PrintResumeKeys STATIC
	SHARED Ega, Green, White, SidebarCol, Called$, Blanks$

	IF Ega THEN COLOR Green
	LOCATE 10, SidebarCol
	PRINT " OPTIONS:   "
	IF Ega THEN COLOR White
	LOCATE , SidebarCol
	PRINT "<c>ontinue  "               'resume the same run
	LOCATE , SidebarCol
	PRINT " this run   "
	LOCATE , SidebarCol
	PRINT "<r>un again "               'start a new run, same parameters
	LOCATE , SidebarCol
	PRINT "<e>dit      "               'edit parameters
	LOCATE , SidebarCol
	PRINT "<i>nitialize"               'initialize
	LOCATE , SidebarCol
	PRINT " all values "
	LOCATE , SidebarCol
	PRINT " to defaults"
	LOCATE , SidebarCol
	IF Called$ = "" THEN               'quit
		PRINT "<q>uit      "
		LOCATE , SidebarCol
		PRINT Blanks$
	ELSE PRINT "<Esc> for   "
		LOCATE , SidebarCol
		PRINT " main menu  "
	END IF
	LOCATE , SidebarCol                'blank row (sparing "Sweep = ")
	PRINT Blanks$

END SUB

SUB RightLabels STATIC
	SHARED Ega, Green, Yellow, White, Mono, SidebarCol
	SHARED XScale, YScale

	CALL ClearMiddle
	CALL ClearBottom
	XStart = 575 / XScale                        'labels for channel
	XEnd = 600 / XScale
    
	LOCATE 17, SidebarCol
	IF Ega THEN COLOR Green
	PRINT "Channel sum"                     'label pseudomacroscopic sum
	LOCATE 18, SidebarCol
	PRINT "   (   )"
	LINE (XStart, 244 / YScale)-(XEnd, 244 / YScale)            'sum in green
	LOCATE 19, SidebarCol
	IF Ega THEN COLOR Yellow
	PRINT "Theoretical"                     'channel open probability yellow
	LOCATE 20, SidebarCol
	PRINT "   (   )"
	IF Mono THEN
		FOR j = 1 TO 12 / XScale
			PSET ((575 / XScale + 2 * j), 272 / YScale)       'dots if mono
		NEXT j
	ELSE LINE (XStart, 272 / YScale)-(XEnd, 272 / YScale)       'else solid
	END IF
	IF Ega THEN COLOR White

END SUB

