''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'                                                                            '
'    SIMULATION OF THE ACTION POTENTIAL OF THE SQUID GIANT AXON              '
'         AFTER HODGKIN & HUXLEY, J. PHYSIOL. 117:500 (1952).                '
'                                                                            '
'    Computer program based on Randall JE, Microcomputers                    '
'         and Physiological Simulation, Addison-Wesley,                      '
'         Reading, MA, pp. 157-198 (1980).                                   '
'                                                                            '
'    Latest revision by Stephen W. Jones, 2/25/89                            '
'    Written for IBM PC-compatibles, using QuickBASIC 4.5.                   '
'    Requires CGA, EGA, Hercules, or AT&T graphics adapter.                  '
'                                                                            '
'    VCLAMP.BAS                                                              '
'                                                                            '
'    This version does voltage clamp.                                        '
'    It displays the membrane potential, and either the                      '
'         Na and K conductances, the open probabilities of the               '
'         n, m, and h gates, or the Na and K currents.                       '
'    Ion concentration ratios can be varied.                                 '
'    Na and K conductances can be reduced by "STX" and "TEA".                '
'    Na inactivation gates can be removed by "pronase".                      '
'    Series resistance can be introduced, and the capacitance can be varied. '
'                                                                            '
'    This program will return control to a menu program that calls           '
'         it via CHAIN.                                                      '
'    The menu program can also allow access to simplified versions,          '
'         with fewer options in a single on-screen menu.                     '
'    Data can be written to either .PRN (text, Lotus-compatible) or          '
'         .DAT (random access binary, CLAMPEX-compatible) files for          '
'         further analysis.                                                  '
'                                                                            '
'    Other modules needed:  EDITVAC, EDITVA, PARAMV                          '
'                                                                            '
'    Include files:                                                          '
'         DECLARES.INC (subprogram declarations and common blocks)           '
'         GRAPHICS.INC (subroutine Graphics and error traps)                 '
'         FILEOUT.INC (subroutines Dimensions, GetFileName,                  '
'              MakeClampexParameters, and MakeClampexLabels)                 '
'         PARAM.INC (subroutines for SUB NewParameters in module PARAMV)     '                         
'                                                                            '
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

'    $INCLUDE: 'declares.inc'

		True = -1                     'allow logical if
		False = NOT (True)
    
	CALL PeekInfo                      'get option, calling program name
	IF Simulation% = 0 THEN            'If no option passed,
		CALL GetCommand                    'get option from command line.
	END IF
	GOSUB Dimensions                   'dimension arrays
	GOSUB Graphics                     'define colors, clear screen
	CALL PrintIntroText                'introductory text
    
INITIAL:
    
	GOSUB Initializations              'initialize variables
	GOSUB MakeParameterArray           'use arrays for easier editing
   
NEXTPLOT:
   
	CALL NewParameters                 'activate menus; allow exit from program
    
		IF ReInit THEN GOTO INITIAL
	
	GOSUB NewVariables                 'get parameters out of arrays
    
		IF ClearScreen OR ClearNext OR FirstScreen THEN CLS 0
		ON ERROR GOTO TRAP            'if error, stop run
    
	CALL Axes                     'draw and label axes
	GOSUB Pharmacology            'STX, TEA, pronase, ion concentrations
	GOSUB FirstPoint              'calculate initial values
	CALL Parameters               'print current parameters as sidebar
	GOSUB AbortRun                'print "hit any key to stop run"

ITERATE:                           'calculate another set of values
	   
		Abort$ = INKEY$
		IF Abort$ = Esc$ THEN
			IF FileOut <> 1 THEN CLOSE #1 'close output file, if any
			GOTO NEXTPLOT       'stop run; what next?
		END IF

	GOSUB Stimulus                'switch to command potential?
	GOSUB Rates                   'rate constants
	GOSUB Gates                   'n, m, and h gates
	GOSUB Currents                'conductances and currents
	GOSUB Plots                   'graph new currents, voltage

		T& = T& + 1                        'increment point #
		IF T& < Tint GOTO ITERATE          'next point or next plot
		IF Measure THEN GOSUB WriteMeasure 'measurement routine results
		IF FileOut <> 1 THEN CLOSE #1      'close output file, if any
	
GOTO NEXTPLOT                      'next run

'  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '

TRAP:
	ErrorType = ERR                         'Main error trap
	IF ErrorType = 6 OR ErrorType = 5 THEN
		SOUND 500, 1                            'Sound a warning
		RESUME NEXTPLOT                         'Error 6 = overflow
	END IF
	ON ERROR GOTO 0

'    $INCLUDE: 'graphics.inc'

'    $INCLUDE: 'fileout.inc'

Initializations:                   'Initialize variables
	ReInit = False
	NextMenu = 0
	FileNumber = 0                     'for naming output files
	SpecialChar$ = "v"                      '"v" for V-clamp
   
	SidebarCol = 69                    'Starting point for sidebar menus
	PrintDefaultAt = 14                 'Column for printing current values
   
	PlotType = 2                       'Display menu parameters
	PlotK = True
	PlotNa = True
	PlotTotal = False
	TimeScale = 4                                'default 10 msec timebase
	Time = 10                                    'default 10 msec timebase
	ClearNext = False
	ClearScreen = False
	FirstScreen = True
  
	HP = -60                           'Stimuli menu parameters
	VStim1 = 0
	Duration1 = 6
	Start1 = 1
	VStim2 = HP
	Duration2 = 0
	Start2 = 0
   
	Stx = 0                            'Pharmacology menu parameters
	Tea = 0
	IntactH = 100
	EKModel = -72                      'Equilibrium potentials on H-H model,
	ENaModel = 55                           'assuming rp = -60
	NaRatio = 1
	KRatio = 1
	RSeries = 0
	Cap = 1
	SlowCalc = False

	FileOut = 1                        'Output menu parameter
    
	gKMax = 36                         'Potassium channel kinetics parameters
	nExp = 4
	Can = .01
	Van0 = -10
	Van = 10
	anExp = False
	Cbn = .125
	Vbn0 = 0
	Vbn = -80

	gNaMax = 120                       'Sodium activation parameters
	mExp = 3
	Cam = .1
	Vam0 = -25
	Vam = 10
	amExp = False
	Cbm = 4
	Vbm0 = 0
	Vbm = -18

	hExp = 1                           'Sodium inactivation parameters
	Cah = .07
	Vah0 = 0
	Vah = -20
	Cbh = 1
	Vbh0 = -30
	Vbh = 10
	bhExp = False
	gLeak = .3                                   'default leak conductance

	RETURN

MakeParameterArray:                     'editing routine uses this array
	Parameter(2, 1) = PlotType
	Parameter(2, 2) = PlotNa
	Parameter(2, 3) = PlotK
	Parameter(2, 4) = PlotTotal
	Parameter(2, 5) = TimeScale
	Parameter(2, 6) = ClearScreen
	Parameter(2, 7) = False

	Parameter(3, 1) = HP
	Parameter(3, 2) = VStim1
	Parameter(3, 3) = Start1
	Parameter(3, 4) = Duration1
	Parameter(3, 5) = VStim2
	Parameter(3, 6) = Start2
	Parameter(3, 7) = Duration2
    
	Parameter(4, 1) = Stx
	Parameter(4, 2) = Tea
	Parameter(4, 3) = IntactH
	Parameter(4, 4) = NaRatio
	Parameter(4, 5) = KRatio
	Parameter(4, 6) = RSeries
	Parameter(4, 7) = Cap
	Parameter(4, 8) = SlowCalc
    
	Parameter(5, 1) = HP
	Parameter(5, 2) = VStim1
	Parameter(5, 3) = Start1
	Parameter(5, 4) = Duration1
	Parameter(5, 5) = VStim2
	Parameter(5, 6) = Start2
	Parameter(5, 7) = Duration2
	Parameter(5, 8) = False
    
	Parameter(6, 1) = HP
	Parameter(6, 2) = VStim1
	Parameter(6, 3) = Start1
	Parameter(6, 4) = Duration1
	Parameter(6, 5) = VStim2
	Parameter(6, 6) = Start2
	Parameter(6, 7) = Duration2
	Parameter(6, 8) = False
    
	Parameter(7, 1) = FileOut
    
	Parameter(8, 1) = StartM(1)
	Parameter(8, 2) = EndM(1)
	Parameter(8, 3) = StartM(2)
	Parameter(8, 4) = EndM(2)
	Parameter(8, 5) = StartM(3)
	Parameter(8, 6) = EndM(3)
	Parameter(8, 7) = StartM(4)
	Parameter(8, 8) = EndM(4)

	Parameter(9, 1) = gKMax
	Parameter(9, 2) = nExp
	Parameter(9, 3) = Can
	Parameter(9, 4) = -60 - Van0            'Change voltages to HH units
	Parameter(9, 5) = Van
	Parameter(9, 6) = anExp
	Parameter(9, 7) = Cbn
	Parameter(9, 8) = -60 - Vbn0
	Parameter(9, 9) = Vbn
    
	Parameter(10, 1) = gNaMax
	Parameter(10, 2) = mExp
	Parameter(10, 3) = Cam
	Parameter(10, 4) = -60 - Vam0
	Parameter(10, 5) = Vam
	Parameter(10, 6) = amExp
	Parameter(10, 7) = Cbm
	Parameter(10, 8) = -60 - Vbm0
	Parameter(10, 9) = Vbm

	Parameter(11, 1) = hExp
	Parameter(11, 2) = Cah
	Parameter(11, 3) = -60 - Vah0
	Parameter(11, 4) = Vah
	Parameter(11, 5) = Cbh
	Parameter(11, 6) = -60 - Vbh0
	Parameter(11, 7) = Vbh
	Parameter(11, 8) = bhExp
	Parameter(11, 9) = gLeak

	RETURN

NewVariables:        'Convert from parameter array to individual variables
	PlotType = Parameter(2, 1)              'Display menu parameters
	PlotNa = Parameter(2, 2)
	PlotK = Parameter(2, 3)
	PlotTotal = Parameter(2, 4)
	TimeScale = Parameter(2, 5)
	ClearScreen = Parameter(2, 6)
	ClearNext = Parameter(2, 7)

	HP = Parameter(3, 1)                    'Stimuli menu parameters
	VStim1 = Parameter(3, 2)
	Start1 = Parameter(3, 3)
	Duration1 = Parameter(3, 4)
	VStim2 = Parameter(3, 5)
	Start2 = Parameter(3, 6)
	Duration2 = Parameter(3, 7)

	Stx = Parameter(4, 1)                   'Pharmacology menu parameters
	Tea = Parameter(4, 2)
	IntactH = Parameter(4, 3)
	NaRatio = Parameter(4, 4)
	KRatio = Parameter(4, 5)
	RSeries = Parameter(4, 6)
	Cap = Parameter(4, 7)
	SlowCalc = Parameter(4, 8)

	FileOut = Parameter(7, 1)               'Output menu parameter
  
	StartM(1) = Parameter(8, 1)             'Measurement menu parameters
	EndM(1) = Parameter(8, 2)
	StartM(2) = Parameter(8, 3)
	EndM(2) = Parameter(8, 4)
	StartM(3) = Parameter(8, 5)
	EndM(3) = Parameter(8, 6)
	StartM(4) = Parameter(8, 7)
	EndM(4) = Parameter(8, 8)
	Measure = False
	FOR i = 1 TO 8
		IF Parameter(8, i) > 0 THEN Measure = True
	NEXT i
    
	gKMax = Parameter(9, 1)                 'Potassium kinetics parameters
	nExp = Parameter(9, 2)
	Can = Parameter(9, 3)
	Van0 = -60 - Parameter(9, 4)
	Van = Parameter(9, 5)
	anExp = Parameter(9, 6)
	Cbn = Parameter(9, 7)
	Vbn0 = -60 - Parameter(9, 8)
	Vbn = Parameter(9, 9)

	gNaMax = Parameter(10, 1)               'Sodium activation parameters
	mExp = Parameter(10, 2)
	Cam = Parameter(10, 3)
	Vam0 = -60 - Parameter(10, 4)
	Vam = Parameter(10, 5)
	amExp = Parameter(10, 6)
	Cbm = Parameter(10, 7)
	Vbm0 = -60 - Parameter(10, 8)
	Vbm = Parameter(10, 9)

	hExp = Parameter(11, 1)                 'Sodium inactivation parameters
	Cah = Parameter(11, 2)
	Vah0 = -60 - Parameter(11, 3)
	Vah = Parameter(11, 4)
	Cbh = Parameter(11, 5)
	Vbh0 = -60 - Parameter(11, 6)
	Vbh = Parameter(11, 7)
	bhExp = Parameter(11, 8)
	gLeak = Parameter(11, 9)

	SELECT CASE Simulation%
		CASE 2
			HP = Parameter(5, 1)               'Conductance menu
			VStim1 = Parameter(5, 2)
			Start1 = Parameter(5, 3)
			Duration1 = Parameter(5, 4)
			VStim2 = Parameter(5, 5)
			Start2 = Parameter(5, 6)
			Duration2 = Parameter(5, 7)
			ClearNext = Parameter(5, 8)
		   
			PlotType = 1
			PlotNa = True
			PlotK = True
			PlotTotal = False
		CASE 3
			HP = Parameter(6, 1)               'Current menu
			VStim1 = Parameter(6, 2)
			Start1 = Parameter(6, 3)
			Duration1 = Parameter(6, 4)
			VStim2 = Parameter(6, 5)
			Start2 = Parameter(6, 6)
			Duration2 = Parameter(6, 7)
			ClearNext = Parameter(6, 8)
		   
			PlotType = 2
			PlotNa = True
			PlotK = True
			PlotTotal = False
		CASE ELSE
	END SELECT

	Time = TimeOption(TimeScale)
	Tint = IntervalOption(TimeScale)
	IF SlowCalc THEN                             'Higher time resolution?
		Tint = 50 * Time                             'at least 50 kHz
		IF Tint < 1000 THEN Tint = 1000              'at least 1000 points
		IF TimeScale = 1 THEN Tint = 2000            'for Na tail currents
	END IF
	Tint = Tint / XScale                         'Adjust if Hercules card
	dt = Time / Tint
	IF PlotNa THEN ZeroI = 195 - 2 * 21          'Adjust zero current level?
	IF PlotK THEN ZeroI = 195 + 3 * 21
	IF PlotTotal OR (PlotNa AND PlotK) THEN ZeroI = 195 + 21

	RETURN

Pharmacology:
	KdStx = 1                                    'dissoc constant = 1 nM
	KdTea = 1                                    'dissoc constant = 1 mM
	UnblockedNa = KdStx / (Stx + KdStx)               'STX
	UnblockedK = KdTea / (Tea + KdTea)                'TEA

	RETURN

FirstPoint:
	T& = 0                                  'T& is the iteration number
	T = T& / Tint                           'this helps avoid overflow
	TPlot% = 1 + XStart + T * 500 / XScale  'used for plotting
	StimOn = False
	FirstScreen = False
    
	Max(1) = 0                              'initialize measurements
	Max(2) = 0                                   '1 and 2 absolute value
	Max(3) = -10 ^ 6                             'detects maximum
	Max(4) = 10 ^ 6                              'detects minimum
   
	gK = 0                        'Initialize:
	gNa = 0
	iT = 0
	mV = HP                            'initial voltage = holding potential
	mVOld = mV

	gKleak = .82205 * gLeak                 'Chosen to give Eleak = -49.4,
	gNaleak = .17795 * gLeak                     'if leak conductance = 0.3
   
	EK = EKModel + 25 * LOG(KRatio)         'RT/nF = 25 mV, natural log
	ENa = ENaModel + 25 * LOG(NaRatio)
	EKHH = -EK - 60                         'convert to H-H units
	ENaHH = -ENa - 60

	Tau = Cap * RSeries / 1000         'time constant of voltage clamp
	IF Tau > 0 THEN
		ExpT = EXP(-dt / Tau)         'used to calculate voltage error
	ELSE ExpT = 0
	END IF

	GOSUB Rates                        'get rate constants

	n = an / (an + bn)                 'steady-state values at initial V
	m = am / (am + bm)
	h = ah / (ah + bh)

	GOSUB Currents                     'calculate currents

	SELECT CASE FileOut                'Write data to files, if requested
		CASE 2                                       '.PRN file (Lotus)
			FileExt$ = ".prn"
			GOSUB GetFileName
			OPEN FileName$ FOR OUTPUT AS #1
			GOSUB MakeLotusHeader
		CASE 3                                       '.DAT file (pCLAMP)
			FileExt$ = ".dat"
			GOSUB GetFileName
		    
			SELECT CASE PlotType
				CASE 1
					OutputGain = 50               'gain [mV/(mS/cm2)]
				CASE 2
					OutputGain = 1000             'gain [mV/(mA/cm2)]
				CASE ELSE
					OutputGain = 10000            'gain [10 V = all open]
			END SELECT
			GOSUB MakeClampexParameters             'Construct header
		    
			Comment$ = "Output from squid axon model"    'Comment
			SELECT CASE PlotType
				CASE 1
					Label$ = "          mS/cm2"   'conducctances
				CASE 2
					Label$ = "          mA/cm2"   'currents
				CASE ELSE
					Label$ = "           gates"   'gates
			END SELECT
			GOSUB MakeClampexLabels                 'Write labels to file
		    
			OPEN FileName$ FOR RANDOM AS #1 LEN = 2      'Data as integers,
			StartPut = 512                                    'start here.
		CASE ELSE
	END SELECT
	
	GOSUB Plots                        'Plot results

	T& = T& + 1                        'Increment point number

	RETURN

MakeLotusHeader:
	SELECT CASE PlotType
		CASE 1                                            'conductances
			IF PlotTotal THEN
				IF PlotNa AND PlotK THEN
					WRITE #1, "TIME", "Voltage", "Na", "K", "TOTAL"
					WRITE #1, "ms", "mV", "mS/cm2", "mS/cm2", "mS/cm2"
				ELSEIF PlotNa THEN
					WRITE #1, "TIME", "Voltage", "Na", "TOTAL"
					WRITE #1, "ms", "mV", "mS/cm2", "mS/cm2"
				ELSEIF PlotK THEN
					WRITE #1, "TIME", "Voltage", "K", "TOTAL"
					WRITE #1, "ms", "mV", "mS/cm2", "mS/cm2"
				ELSE
					WRITE #1, "TIME", "Voltage", "TOTAL"
					WRITE #1, "ms", "mV", "mS/cm2"
				END IF
			ELSEIF PlotNa AND PlotK THEN
				WRITE #1, "TIME", "Voltage", "Na", "K"
				WRITE #1, "ms", "mV", "mS/cm2", "mS/cm2"
			ELSEIF PlotNa THEN
				WRITE #1, "TIME", "Voltage", "Na"
				WRITE #1, "ms", "mV", "mS/cm2"
			ELSEIF PlotK THEN
				WRITE #1, "TIME", "Voltage", "K"
				WRITE #1, "ms", "mV", "mS/cm2"
			ELSE
				WRITE #1, "TIME", "Voltage", "Na", "K"
				WRITE #1, "ms", "mV", "mS/cm2", "mS/cm2"
			END IF
		CASE 2                                            'currents
			IF PlotTotal THEN
				IF PlotNa AND PlotK THEN
					WRITE #1, "TIME", "Voltage", "Na", "K", "TOTAL"
					WRITE #1, "ms", "mV", "mA/cm2", "mA/cm2", "mA/cm2"
				ELSEIF PlotNa THEN
					WRITE #1, "TIME", "Voltage", "Na", "TOTAL"
					WRITE #1, "ms", "mV", "mA/cm2", "mA/cm2"
				ELSEIF PlotK THEN
					WRITE #1, "TIME", "Voltage", "K", "TOTAL"
					WRITE #1, "ms", "mV", "mA/cm2", "mA/cm2"
				ELSE
					WRITE #1, "TIME", "Voltage", "TOTAL"
					WRITE #1, "ms", "mV", "mA/cm2"
				END IF
			ELSEIF PlotNa AND PlotK THEN
				WRITE #1, "TIME", "Voltage", "Na", "K"
				WRITE #1, "ms", "mV", "mA/cm2", "mA/cm2"
			ELSEIF PlotNa THEN
				WRITE #1, "TIME", "Voltage", "Na"
				WRITE #1, "ms", "mV", "mA/cm2"
			ELSEIF PlotK THEN
				WRITE #1, "TIME", "Voltage", "K"
				WRITE #1, "ms", "mV", "mA/cm2"
			END IF
		CASE 3                                            'gates
			IF PlotNa AND PlotK THEN
				WRITE #1, "TIME", "Voltage", "Na (m)", "Na (h)", "K (n)"
			ELSEIF PlotNa THEN
				WRITE #1, "TIME", "Voltage", "Na (m)", "Na (h)"
			ELSE
				WRITE #1, "TIME", "Voltage", "K (n)"
			END IF
			WRITE #1, "ms"
		CASE ELSE
	END SELECT

	RETURN

AbortRun:
	IF Ega THEN COLOR Red                   'print in red
	IF Simulation% >= 2 AND Simulation% <= 3 THEN     'simplified versions
		LOCATE 18, SidebarCol
		PRINT " HIT <Esc>  "
		LOCATE , SidebarCol
		PRINT "TO STOP RUN "
	ELSE
		LOCATE 22, SidebarCol                   'conserve space otherwise
		PRINT "<Esc> = STOP"
	END IF

	RETURN                                  'main program checks for input

Stimulus:
	mVOld = mV
	VssError = iT * RSeries / 1000          'steady-state clamp error
	OnOrOff = False
	OldStim = StimOn
	StimOn = False
   
	IF T& > Start1 / dt - .1 AND T& < -.1 + (Start1 + Duration1) / dt THEN
		mVss = VStim1 + VssError
		StimOn = True
	END IF
	IF T& > Start2 / dt - .1 AND T& < -.1 + (Start2 + Duration2) / dt THEN
		mVss = VStim2 + VssError
		StimOn = True
	END IF
	IF OldStim <> StimOn THEN OnOrOff = True     'checks for stim on or off

	IF NOT (StimOn) THEN mVss = HP + VssError    'if no command, reset to HP

	mV = mVss + (mVOld - mVss) * ExpT            'delay introduced by Rs*C

	RETURN

Rates:
	V = -mV - 60                       'set rp = -60 mV.  V is in H-H units
	IF V - Van0 = 0 THEN V = V - .01         'avoid division by zero
	IF V - Vam0 = 0 THEN V = V - .01

	IF anExp THEN
		an = Can * EXP(-(V - Van0) / Van)
	ELSE an = Can * (V - Van0) / (EXP((V - Van0) / Van) - 1)    'a=opening
	END IF
	IF amExp THEN
		am = Cam * EXP(-(V - Vam0) / Vam)
	ELSE am = Cam * (V - Vam0) / (EXP((V - Vam0) / Vam) - 1)
	END IF
	ah = Cah * EXP(-(V - Vah0) / Vah)
    
	bn = Cbn * EXP(-(V - Vbn0) / Vbn)                           'b=closing
	bm = Cbm * EXP(-(V - Vbm0) / Vbm)
	IF bhExp THEN
		bh = Cbh * EXP(-(V - Vbh0) / Vbh)
	ELSE bh = Cbh / (EXP((V - Vbh0) / Vbh) + 1)
	END IF

	RETURN

Gates:
	dn = an * (1 - n) - bn * n         'increments
	dm = am * (1 - m) - bm * m
	dh = ah * (1 - h) - bh * h

	n = n + dn * dt                    'current values of gate p(open)
	m = m + dm * dt
	h = h + dh * dt

	IF n > 1 THEN n = 1                'recover from extrapolation errors
	IF m > 1 THEN m = 1
	IF h > 1 THEN h = 1
	IF n < 0 THEN n = 0
	IF m < 0 THEN m = 0
	IF h < 0 THEN h = 0

	RETURN

Currents:
	NetH = ((IntactH * h + (100 - IntactH)) / 100) ^ hExp       'pronase
   
	IF PlotK OR NOT PlotNa THEN                  'Conductance units: mS/cm2
		gK = UnblockedK * gKMax * n ^ nExp                     'n to 4th
	END IF
	IF PlotNa OR NOT PlotK THEN
		gNa = UnblockedNa * gNaMax * m ^ mExp * NetH           'm cubed h
	END IF
	gT = gK + gNa + gLeak

	iK = gK * (V - EKHH)                         'Calculate currents (A/cm2)
	iNa = gNa * (V - ENaHH)
	iL = gKleak * (V - EKHH) + gNaleak * (V - ENaHH)  'parallel batteries
	iT = iK + iNa + iL
	iC = Cap * (mVOld - mV) / dt

	RETURN

Plots:
	PlotDot = False
	T = T& / Tint                           'This helps avoid overflow
	ms = T * Time                           'Time in ms
	TPlot% = 1 + XStart + T * 500 / XScale  'Convert time to screen units
	IF T& > 0 AND T& <= 50 THEN
		PlotDot = True
		TDot = XStart + T& * 10 / XScale    'Used for dotted lines
	END IF

	IF Ega THEN COLOR Yellow                'Plot voltage commands:
	PSET (TPlot%, (41 - .4 * mV) / YScale)       '0 mV at 41; 40 pts/100 mV
	IF Ega THEN COLOR White

	SELECT CASE PlotType                    'Plot:
		CASE 1
			GOSUB PlotConductances             'conductances,
		CASE 2
			GOSUB PlotCurrents                 'currents, or
		CASE ELSE
			GOSUB PlotGates                    'gates.
	END SELECT

	SELECT CASE FileOut                     'Write to output file (if any)
		CASE 2
			GOSUB WriteToLotus
		CASE 3
			GOSUB WriteToPClamp
		CASE ELSE
	END SELECT

	IF Measure THEN GOSUB Measurements
    
	RETURN

WriteToLotus:                                     'Lotus-type file
	SELECT CASE PlotType
		CASE 1                                       'conductances
			IF PlotTotal THEN
				IF PlotNa AND PlotK THEN
					WRITE #1, ms, mV, gNa, gK, gT
				ELSEIF PlotNa THEN
					WRITE #1, ms, mV, gNa, gT
				ELSEIF PlotK THEN
					WRITE #1, ms, mV, gK, gT
				ELSE
					WRITE #1, ms, mV, gT
				END IF
			ELSEIF PlotNa AND PlotK THEN
				WRITE #1, ms, mV, gNa, gK
			ELSEIF PlotNa THEN
				WRITE #1, ms, mV, gNa
			ELSEIF PlotK THEN
				WRITE #1, ms, mV, gK
			ELSE
				WRITE #1, ms, mV, gNa, gK
			END IF
		CASE 2                                       'currents
			mANa = -iNa / 1000
			mAK = -iK / 1000
			mAT = -(iT + iC) / 1000

			IF PlotTotal THEN
				IF PlotNa AND PlotK THEN
					WRITE #1, ms, mV, mANa, mAK, mAT
				ELSEIF PlotNa THEN
					WRITE #1, ms, mV, mANa, mAT
				ELSEIF PlotK THEN
					WRITE #1, ms, mV, mAK, mAT
				ELSE
					WRITE #1, ms, mV, mAT
				END IF
			ELSEIF PlotNa AND PlotK THEN
				WRITE #1, ms, mV, mANa, mAK
			ELSEIF PlotNa THEN
				WRITE #1, ms, mV, mANa
			ELSEIF PlotK THEN
				WRITE #1, ms, mV, mAK
			ELSE
				WRITE #1, ms, mV, mANa, mAK
			END IF
		CASE 3                                       'gates
			IF PlotNa AND PlotK THEN
				WRITE #1, ms, mV, m, h, n
			ELSEIF PlotNa THEN
				WRITE #1, ms, mV, m, h
			ELSE
				WRITE #1, ms, mV, n
			END IF
		CASE ELSE
	END SELECT

	RETURN

WriteToPClamp:                                    'CLAMPEX-type file
	SELECT CASE PlotType
		CASE 1                                       'conductance
			IF PlotTotal THEN
				Value = gT
			ELSEIF PlotNa THEN
				Value = gNa
			ELSE Value = gK
			END IF
		CASE 2                                       'current
			IF PlotTotal THEN
				Value = -(iT + iC) / 1000
			ELSEIF PlotNa THEN
				Value = -iNa / 1000
			ELSE Value = -iK / 1000
			END IF
		CASE 3                                       'gate
			IF PlotNa THEN
				IF PlotTotal THEN
					Value = h
				ELSE Value = m
				END IF
			ELSE Value = n
			END IF
		CASE ELSE
			Value = 0
	END SELECT

	ScaledValue = Value * 2.048 * OutputGain / 10
	IF ScaledValue > 2048 THEN ScaledValue = 2048
	IF ScaledValue < -2048 THEN ScaledValue = -2048
	ClampexData = CINT(ScaledValue)                   'convert to integer
	StartPut = StartPut + 1
	IF T& < PointsToWrite THEN
		PUT #1, StartPut, ClampexData                'write to file
	END IF
	IF T& = 0 THEN                                    'duplicate 1st value
		FOR i = 1 TO FirstPoints
			StartPut = StartPut + 1
			PUT #1, StartPut, ClampexData
		NEXT i
	END IF
  
	RETURN

PlotConductances:
	IF PlotNa THEN
		IF Ega THEN COLOR Blue
		PSET (TPlot%, (300 - 1.75 * gNa) / YScale)   'Na conductance, blue
		IF PlotDot THEN
			IF Ega THEN COLOR White
			PSET (TDot, 90 / YScale)                'dotted line at G max Na
		END IF
	END IF
	IF PlotK THEN
		IF HiRes = 9 OR TPlot% / 2 = INT(TPlot% / 2) THEN
			IF Ega THEN COLOR Green
			PSET (TPlot%, (300 - 1.75 * gK) / YScale)    'K cond., green
		END IF
		IF PlotDot THEN
			IF Ega THEN COLOR White
			PSET (TDot, 237 / YScale)               'dotted line at G max K
		END IF
	END IF
	RETURN

PlotCurrents:
	IF Ega THEN COLOR White
	IF PlotDot THEN PSET (TDot, ZeroI / YScale)       'dotted line at I = 0
	IF PlotTotal THEN
		PlotValue = (ZeroI + (iT + iC) * .021) / YScale     'avoid overflow
		IF ABS(PlotValue) < 1000 THEN
			PSET (TPlot%, PlotValue)                'Total current, or:
		END IF
	END IF
	IF PlotNa THEN
		IF Ega THEN COLOR Blue                       'Na current, blue
		PSET (TPlot%, (ZeroI + iNa * .021) / YScale)
	END IF
	IF PlotK AND (HiRes = 9 OR TPlot% / 2 = INT(TPlot% / 2)) THEN
		IF Ega THEN COLOR Green                      'K current, green
		PSET (TPlot%, (ZeroI + iK * .021) / YScale)
	END IF

	RETURN

PlotGates:
	IF PlotNa THEN
		IF Ega THEN COLOR Blue                       'm gate in blue
		PSET (TPlot%, 300 - 210 * m)
		IF HiRes = 9 OR TPlot% / 2 = INT(TPlot% / 2) THEN
			IF Ega THEN COLOR Red
			PSET (TPlot%, 300 - 210 * NetH)         'h gate in red
		END IF
	END IF
	IF PlotK THEN
		IF Ega THEN COLOR Green                      'n gate in green
		PSET (TPlot%, 300 - 210 * n)
	END IF
   
	IF Ega THEN COLOR White
	IF PlotDot THEN PSET (TDot, 90 / YScale)          'dotted line at open
   
	RETURN

Measurements:                                'Measure one thing per sweep
	SELECT CASE PlotType                         'See WriteToPClamp
		CASE 1                                       'conductance
			Unit$ = "mS/cm2"
			IF PlotTotal THEN
				Value = gT
				Type$ = "Total"
			ELSEIF PlotNa THEN
				Value = gNa
				Type$ = "Na"
			ELSE
				Value = gK
				Type$ = "K"
			END IF
		CASE 2                                       'current
			Unit$ = "mA/cm2"
			IF PlotTotal THEN
				Value = -(iT + iC) / 1000
				Type$ = "Total"
			ELSEIF PlotNa THEN
				Value = -iNa / 1000
				Type$ = "Na"
			ELSE
				Value = -iK / 1000
				Type$ = "K"
			END IF
		CASE 3                                       'gate
			Unit$ = "gate"
			IF PlotNa THEN
				IF PlotTotal THEN
					Value = h
					Type$ = "h"
				ELSE
					Value = m
					Type$ = "m"
				END IF
			ELSE
				Value = n
				Type$ = "n"
			END IF
		CASE ELSE
			Value = 0
			Type$ = ""
	END SELECT

	FOR i = 1 TO 4                                    '4 measurements
		IF ms >= StartM(i) AND ms < EndM(i) THEN
			SELECT CASE i
				CASE 1 - 2
					IF ABS(Value) > ABS(Max(i)) THEN Max(i) = Value
				CASE 3
					IF Value > Max(i) THEN Max(i) = Value
				CASE ELSE
					IF Value < Max(i) THEN Max(i) = Value
			END SELECT
		END IF
		IF ms = StartM(i) AND ms = EndM(i) THEN
			Max(i) = Value
		END IF
	NEXT i
    
	RETURN

WriteMeasure:                                          'measurements to file
	OPEN "Measure.Prn" FOR APPEND AS #2
	WRITE #2, Type$, Unit$, Max(1), Max(2), Max(3), Max(4)
	CLOSE #2

	RETURN

SUB Axes STATIC
	SHARED Ega, Mono, HiRes, White, Red, Blue, Green
	SHARED XStart, YStart, XScale, YScale
	SHARED PlotType, Time

	IF Ega THEN COLOR White
	CALL ClearLeft
	CALL ClearBottom

	XStart = 40 / XScale
	XTick = 34 / XScale
	XEnd = 540 / XScale

	YStart = 1 / YScale
	YEnd = 81 / YScale
	Y2Start = 90 / YScale
	Y2End = 300 / YScale
	YTick = 306 / YScale

	LINE (XStart, Y2End)-(XEnd, Y2End)       'time axis
	LINE (XStart, YStart)-(XStart, YEnd)     'voltage axis
	LINE (XStart, Y2Start)-(XStart, Y2End)   'current axis

	X = XStart
	Y = YStart
	Y2 = Y2Start
	FOR i = 1 TO 11                                   'draw tick marks
		IF i < 4 THEN LINE (XStart, Y)-(XTick, Y)         '10 mV, vert
		IF PlotType = 2 THEN
			LINE (XStart, Y2)-(XTick, Y2)                '1 mA/cm2
		END IF
		LINE (X, Y2End)-(X, YTick)                        '1 msec, horiz
		X = X + 50 / XScale                          '1 msec = 50 units
		Y = Y + 40 / YScale                          '100 mV = 40 units
		Y2 = Y2 + 21 / YScale                        '1 mA/cm2 = 21 units
	NEXT i

	LOCATE 23, 6                       '12.5 horiz text units = 20% max
	PRINT "0";
	LOCATE , 18
	CALL PrintNum(Time * .2)
	LOCATE , 31
	CALL PrintNum(Time * .4)
	LOCATE , 43
	CALL PrintNum(Time * .6)
	LOCATE , 56
	CALL PrintNum(Time * .8)
	LOCATE , 68
	CALL PrintNum(Time)
	PRINT " ms";

	LOCATE 1, 1                                  'label voltage axis
	PRINT "100"
	LOCATE 3, 1                                       '100 mV = 2.5 rows
	PRINT "  0"
	LOCATE 4, 2
	PRINT "mV"
	LOCATE 6, 1
	PRINT "-100"

	SELECT CASE PlotType
		CASE 1
			GOSUB LabelConductances            'label main display area
		CASE 2
			GOSUB LabelCurrents
		CASE ELSE
			GOSUB LabelGates
	END SELECT

	EXIT SUB

LabelConductances:
	IF Ega THEN COLOR Blue                       'Na conductance light blue
	LOCATE 7, 3
	PRINT "Na"
	PRINT " (  )"
	LINE (15 / XScale, 104 / YScale)-(31 / XScale, 104 / YScale)
	IF Ega THEN COLOR Green                      'K conductance light green
	LOCATE 17, 4
	PRINT "K"
	PRINT " (  )"
	IF Mono THEN
		FOR j = 1 TO 10 / XScale
			PSET (14 + 2 * j, 244 / YScale)    'dotted line on monochrome
		NEXT j
	ELSE LINE (16 / XScale, 244 / YScale)-(31 / XScale, 244 / YScale)
	END IF

	IF Ega THEN COLOR White                      'reset color to bright white
	LOCATE 10, 1
	PRINT "C"                                    'label conductance
	PRINT "o"
	PRINT "n"
	PRINT "d"
	PRINT "u"
	PRINT "c"
	PRINT "t"
	PRINT "a"
	PRINT "n"
	PRINT "c"
	PRINT "e"

	RETURN

LabelCurrents:
	IF Ega THEN COLOR Blue                       'Na current light blue
	LOCATE 19, 2
	PRINT "Na"
	PRINT "(  )"
	LINE (7 / XScale, 272 / YScale)-(23 / XScale, 272 / YScale)
	IF Ega THEN COLOR Green                      'K current light green
	LOCATE 8, 2
	PRINT "K"
	PRINT "(  )"
	IF Mono THEN                                 'dotted line on monochrome
		FOR j = 1 TO 10 / XScale
			PSET ((5 + 2 * j), 118 / YScale)
		NEXT j
	ELSE LINE (7 / XScale, 118 / YScale)-(23 / XScale, 118 / YScale)
	END IF
	IF Ega THEN COLOR White                      'reset color to bright white
	LOCATE 11, 1
	PRINT " C"                                   'label current axis
	PRINT " u"
	PRINT " r"
	PRINT " r"
	PRINT " e"
	PRINT " n"
	PRINT " t"

	RETURN

LabelGates:
	LOCATE 14, 2
	IF Ega THEN COLOR Red
	PRINT "h"
	PRINT "(  )"
	IF Mono THEN
		FOR j = 1 TO 10 / XScale
			PSET (5 + 2 * j, 202 / YScale)     'dotted line if mono
		NEXT j
	ELSE LINE (7 / XScale, 202 / YScale)-(23 / XScale, 202 / YScale)
	END IF
	LOCATE 20, 2
	IF Ega THEN COLOR Blue
	PRINT "m"
	PRINT "(  )"
	LINE (7 / XScale, 286 / YScale)-(23 / XScale, 286 / YScale)
	LOCATE 17, 2
	IF Ega THEN COLOR Green
	PRINT "n"
	PRINT "(  )"
	LINE (7 / XScale, 244 / YScale)-(23 / XScale, 244 / YScale)

	IF Ega THEN COLOR White
	LOCATE 8, 1
	PRINT " G"
	PRINT " a"
	PRINT " t"
	PRINT " e"
	PRINT " s"

	RETURN

END SUB

SUB GetCommand STATIC              'Check command line for simulation number
	SHARED Simulation%
   
	Option$ = COMMAND$
	Sim = VAL(Option$)
	SELECT CASE Sim
		CASE 2 TO 3                   '2 and 3 are simplified versions
			Simulation% = Sim
		CASE ELSE
	END SELECT

END SUB

SUB Parameters STATIC         'Display current parameters as a sidebar
	SHARED Simulation%
	SHARED Ega, White, Red, Green
	SHARED SidebarCol
	SHARED Parameter()

	CALL ClearRight
	SELECT CASE Simulation%
		CASE 2
			GOSUB ConductanceParameters
		CASE 3
			GOSUB CurrentParameters
		CASE ELSE
			GOSUB AdvancedParameters
	END SELECT

	EXIT SUB

ConductanceParameters:                  'top part of sidebar
	IF Ega THEN COLOR Green

	LOCATE 1, SidebarCol
	PRINT " INITIAL"
	LOCATE 2, SidebarCol
	IF Ega THEN COLOR White
	PRINT "mV=";
	CALL PrintNum(Parameter(5, 1))
	PRINT
	LOCATE 3, SidebarCol
	IF Ega THEN COLOR Green
	PRINT " COMMAND 1  "
	LOCATE 4, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	IF Ega THEN COLOR White
	PRINT "mV=";
	CALL PrintNum(Parameter(5, 2))
	LOCATE 5, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	PRINT "start=";
	CALL PrintNum(Parameter(5, 3))
	LOCATE 6, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	PRINT "msec=";
	CALL PrintNum(Parameter(5, 4))
	PRINT
	LOCATE 7, SidebarCol
	IF Ega THEN COLOR Green
	PRINT " COMMAND 2  "
	LOCATE 8, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	IF Ega THEN COLOR White
	PRINT "mV=";
	CALL PrintNum(Parameter(5, 5))
	LOCATE 9, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	PRINT "start=";
	CALL PrintNum(Parameter(5, 6))
	LOCATE 10, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	PRINT "msec=";
	CALL PrintNum(Parameter(5, 7))

	RETURN

CurrentParameters:
	IF Ega THEN COLOR Green

	LOCATE 1, SidebarCol
	PRINT " INITIAL"
	LOCATE 2, SidebarCol
	IF Ega THEN COLOR White
	PRINT "mV=";
	CALL PrintNum(Parameter(6, 1))
	LOCATE 3, SidebarCol
	IF Ega THEN COLOR Green
	PRINT " COMMAND 1  "
	LOCATE 4, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	IF Ega THEN COLOR White
	PRINT "mV=";
	CALL PrintNum(Parameter(6, 2))
	LOCATE 5, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	PRINT "start=";
	CALL PrintNum(Parameter(6, 3))
	LOCATE 6, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	PRINT "msec=";
	CALL PrintNum(Parameter(6, 4))
	PRINT
	LOCATE 7, SidebarCol
	IF Ega THEN COLOR Green
	PRINT " COMMAND 2  "
	LOCATE 8, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	IF Ega THEN COLOR White
	PRINT "mV=";
	CALL PrintNum(Parameter(6, 5))
	LOCATE 9, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	PRINT "start=";
	CALL PrintNum(Parameter(6, 6))
	LOCATE 10, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	PRINT "msec=";
	CALL PrintNum(Parameter(6, 7))

	RETURN

AdvancedParameters:                               'Full sidebar for advanced
	IF Ega THEN COLOR Green

	LOCATE 1, SidebarCol
	PRINT " INITIAL"
	LOCATE 2, SidebarCol
	IF Ega THEN COLOR White
	PRINT "mV=";
	CALL PrintNum(Parameter(3, 1))
	LOCATE 3, SidebarCol
	IF Ega THEN COLOR Green
	PRINT " COMMAND 1  "
	LOCATE 4, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	IF Ega THEN COLOR White
	PRINT "mV=";
	CALL PrintNum(Parameter(3, 2))
	LOCATE 5, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	PRINT "start=";
	CALL PrintNum(Parameter(3, 3))
	LOCATE 6, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	PRINT "msec=";
	CALL PrintNum(Parameter(3, 4))
	LOCATE 7, SidebarCol
	IF Ega THEN COLOR Green
	PRINT " COMMAND 2  "
	LOCATE 8, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	IF Ega THEN COLOR White
	PRINT "mV=";
	CALL PrintNum(Parameter(3, 5))
	LOCATE 9, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	PRINT "start=";
	CALL PrintNum(Parameter(3, 6))
	LOCATE 10, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	PRINT "msec=";
	CALL PrintNum(Parameter(3, 7))

	LOCATE 11, SidebarCol
	IF Ega THEN COLOR Green
	PRINT " STX CONC.  "
	LOCATE , SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	IF Ega THEN COLOR White
	PRINT "(nM)=";
	CALL PrintNum(Parameter(4, 1))
	LOCATE 13, SidebarCol
	IF Ega THEN COLOR Green
	PRINT " TEA CONC.  "
	LOCATE , SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	IF Ega THEN COLOR White
	PRINT "(mM)=";
	CALL PrintNum(Parameter(4, 2))
	LOCATE 15, SidebarCol
	IF Ega THEN COLOR Green
	PRINT "% H=";
	IF Ega THEN COLOR White
	CALL PrintNum(Parameter(4, 3))
	LOCATE 16, SidebarCol
	IF Ega THEN COLOR Green
	PRINT " Na CONC.   "
	LOCATE , SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	IF Ega THEN COLOR White
	PRINT "RATIO=";
	CALL PrintNum(Parameter(4, 4))
	LOCATE 18, SidebarCol
	IF Ega THEN COLOR Green
	PRINT " K CONC.    "
	LOCATE , SidebarCol
	IF Ega THEN COLOR White
	PRINT "RATIO=";
	CALL PrintNum(Parameter(4, 5))
	LOCATE , SidebarCol
	PRINT
	LOCATE , SidebarCol
	IF Ega THEN COLOR Green
	PRINT "Rs=";
	IF Ega THEN COLOR White
	CALL PrintNum(Parameter(4, 6))
	LOCATE , SidebarCol
	PRINT
	LOCATE , SidebarCol
	IF Ega THEN COLOR Green
	PRINT "Cap=";
	IF Ega THEN COLOR White
	CALL PrintNum(Parameter(4, 7))

	IF Ega THEN COLOR Red
	LOCATE 22, SidebarCol
	PRINT "<F1> = MENUS"
	IF Ega THEN COLOR White

	RETURN

END SUB

SUB PrintIntroText STATIC
	SHARED Simulation%, Called$
	SHARED Ega, White, Red, Blue, Green
 
	IF Ega THEN COLOR Blue
	LOCATE 1, 7
	PRINT "AXOVACS Electrophysiology Computer Lab - Version 2.0"
	LOCATE , 17
	PRINT "(C) 1989 Axon Instruments, Inc."
	PRINT
    
	SELECT CASE Simulation%
		CASE 2
			GOSUB ConductanceText
		CASE 3
			GOSUB CurrentText
		CASE ELSE
			GOSUB AdvancedText
	END SELECT

EXIT SUB

AdvancedText:
	LOCATE , 21
	IF Ega THEN COLOR Red
	PRINT "VOLTAGE CLAMP SIMULATION"
	IF Ega THEN COLOR White
	PRINT
	PRINT "YOU can choose the holding potential,"
	PRINT "        enter parameters for voltage steps (two per run),"
	PRINT "        choose to see either Na and K gates (n, m, and h),"
	PRINT "             Na and K conductances, or Na and K currents,"
	PRINT "        change the Na and/or K conductances by adding"
	PRINT "             simulated drugs (TEA, TTX, and pronase), and"
	PRINT "        change the Na and/or K concentrations."
	PRINT "THE COMPUTER will calculate and display"
	PRINT "        the membrane potential,"
	PRINT "        the gates, conductances, or currents, and"
	PRINT "        the timing of the stimuli."
	PRINT
	IF Ega THEN COLOR Green
	PRINT "The parameters that you can modify are organized in 8 menus:"
	IF Ega THEN COLOR White
	PRINT "DISPLAY:          plots, time scale, superimpose or erase"
	PRINT "STIMULI:          holding potential, voltage clamp steps"
	PRINT "PHARMACOLOGY:     drugs, changes in Na and/or K concentrations"
	PRINT "OUTPUTS, MEASUREMENTS:           write data to files"
	PRINT "n, m, or h-gate KINETICS:        change model parameters"

	RETURN
    
ConductanceText:
	PRINT
	LOCATE , 21
	IF Ega THEN COLOR Red
	PRINT "VOLTAGE CLAMP SIMULATION"
	IF Ega THEN COLOR White
	PRINT
	PRINT
	PRINT "YOU can choose the holding potential, and"
	PRINT "        enter parameters for voltage steps (two per run)."
	PRINT
	PRINT "THE COMPUTER will calculate and display"
	PRINT "        the membrane potential,"
	PRINT "        the Na and K conductances, and"
	PRINT "        the timing of the stimuli."
	PRINT

	GOSUB PrintBottomText

	RETURN

CurrentText:
	PRINT
	LOCATE , 21
	IF Ega THEN COLOR Red
	PRINT "VOLTAGE CLAMP SIMULATION"
	IF Ega THEN COLOR White
	PRINT
	PRINT
	PRINT "YOU can choose the holding potential, and"
	PRINT "        enter parameters for voltage steps (two per run)."
	PRINT
	PRINT "THE COMPUTER will calculate and display"
	PRINT "        the membrane potential,"
	PRINT "        the Na and K currents, and"
	PRINT "        the timing of the stimuli."
	PRINT
    
	GOSUB PrintBottomText

	RETURN

PrintBottomText:
	LOCATE 16
	IF Ega THEN COLOR Green
	PRINT "Hit one of three keys:"
	IF Ega THEN COLOR White
	PRINT "     <r>   to run a simulation with the current parameters"
	PRINT "     <e>   to edit the parameters"
	IF Called$ <> "" THEN
		PRINT "     <Esc> to return to the main menu"
	ELSE PRINT "     <q>   to quit"
	END IF

	RETURN
    
END SUB

